@@ -88,7 +88,7 @@ class IProcessor(Interface):
 
         # implementations should always call the parent method
 
-    def mediainfo():
+    def mediainfo(self):
         """
         Information about the media object
         uri
@@ -96,6 +96,11 @@ class IProcessor(Interface):
         duration
         """
 
+    @staticmethod
+    def uuid():
+        """Return the UUID of the processor"""
+
+
 class IEncoder(IProcessor):
     """Encoder driver interface. Each encoder is expected to support a specific
     format."""
