@@ -29,18 +29,6 @@ class FlacEncoder(GstEncoder):
     """ gstreamer-based FLAC encoder """
     implements(IEncoder)
 
-    def __init__(self, output,  streaming=False):
-        if isinstance(output, basestring):
-            self.filename = output
-        else:
-            self.filename = None
-        self.streaming = streaming
-
-        if not self.filename and not self.streaming:
-            raise Exception('Must give an output')
-
-        self.eod = False
-
     @interfacedoc
     def setup(self, channels=None, samplerate=None, blocksize=None, totalframes=None):
         super(FlacEncoder, self).setup(channels, samplerate, blocksize, totalframes)
