@@ -1,4 +1,4 @@
-    # -*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2007-2012 Parisson SARL
 # Copyright (c) 2006-2012 Guillaume Pellerin <pellerin@parisson.com>
@@ -24,7 +24,7 @@
 # Authors: Guillaume Pellerin <yomguy@parisson.com>
 #          Paul Brossier <piem@piem.org>
 
-from timeside.core import Processor, implements, interfacedoc
+from timeside.core import implements, interfacedoc
 from timeside.encoder.core import GstEncoder
 from timeside.api import IEncoder
 from timeside.tools import *
